@@ -181,7 +181,23 @@ public class PluginAppletViewer extends XEmbeddedFrame
 
 
     private SplashPanel splashPanel;
- 
+    
+    private static long REQUEST_TIMEOUT=60000;//60s
+
+    private static void waitForRequestCompletion(PluginCallRequest request) {
+        try {
+            if (!request.isDone()) {
+                request.wait(REQUEST_TIMEOUT);
+            }
+            if (!request.isDone()) {
+                // Do not wait indefinitely to avoid the potential of deadlock
+                throw new RuntimeException("Possible deadlock, releasing");
+            }
+        } catch (InterruptedException ex) {
+            throw new RuntimeException("Interrupted waiting for call request.", ex);
+        }
+    }
+
     /**
      * Null constructor to allow instantiation via newInstance()
      */
@@ -665,18 +681,7 @@ public class PluginAppletViewer extends XEmbeddedFrame
         if (message.startsWith("width")) {
 
             // Wait for panel to come alive
-            long maxTimeToSleep = APPLET_TIMEOUT;
-            statusLock.lock();
-            try {
-                while (!status.get(identifier).equals(PAV_INIT_STATUS.INIT_COMPLETE) &&
-                        maxTimeToSleep > 0) {
-                    maxTimeToSleep -= waitTillTimeout(statusLock, initComplete,
-                                                      maxTimeToSleep);
-                }
-            }
-            finally {
-                statusLock.unlock();
-            }
+            waitForAppletInit(panel);
 
             // 0 => width, 1=> width_value, 2 => height, 3=> height_value
             String[] dimMsg = message.split(" ");
@@ -1301,18 +1306,13 @@ public class PluginAppletViewer extends XEmbeddedFrame
 
         streamhandler.postCallRequest(request);
         streamhandler.write(request.getMessage());
-        try {
-            PluginDebug.debug("wait ToString request 1");
-            synchronized (request) {
-                PluginDebug.debug("wait ToString request 2");
-                while (request.isDone() == false)
-                    request.wait();
-                PluginDebug.debug("wait ToString request 3");
-            }
-        } catch (InterruptedException e) {
-            throw new RuntimeException("Interrupted waiting for call request.",
-                                        e);
+        PluginDebug.debug("wait ToString request 1");
+        synchronized (request) {
+            PluginDebug.debug("wait ToString request 2");
+            waitForRequestCompletion(request);
+            PluginDebug.debug("wait ToString request 3");
         }
+
         PluginDebug.debug(" ToString DONE");
         return (String) request.getObject();
     }
@@ -1538,7 +1538,9 @@ public class PluginAppletViewer extends XEmbeddedFrame
 
         // If the image or the graphics don't exist, create new ones
         if (bufFrameImg == null || bufFrameImgGraphics == null) {
-            bufFrameImg = createImage(getWidth(), getHeight());
+            // although invisible applets do not have right to paint
+            // we rather paint to 1x1 to be sure all callbacks  will be completed
+            bufFrameImg = createImage(Math.max(1, getWidth()), Math.max(1, getHeight()));
             bufFrameImgGraphics = bufFrameImg.getGraphics();
         }
 
